@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2020 the xine project
+ * Copyright (C) 2000-2021 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -72,7 +72,7 @@ typedef struct {
   int              sync_state;
   int              ac5_length, ac5_pcm_length, frame_todo;
   uint32_t         syncdword;
-  uint8_t          frame_buffer[MAX_AC5_FRAME];
+  uint8_t          frame_buffer[MAX_AC5_FRAME + 1];
   uint8_t         *frame_ptr;
 
   int              output_open;
@@ -474,6 +474,8 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
   lprintf("open_plugin\n");
 
   this = calloc(1, sizeof (dts_decoder_t));
+  if (!this)
+    return NULL;
 
   this->audio_decoder.decode_data         = dts_decode_data;
   this->audio_decoder.reset               = dts_reset;
