@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2018 the xine project
+ * Copyright (C) 2000-2021 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -55,7 +55,7 @@ typedef struct jack_driver_s
   int		paused;
   int		underrun;
 
-  int32_t	output_sample_rate, input_sample_rate;
+  uint32_t	output_sample_rate, input_sample_rate;
   uint32_t	num_channels;
   uint32_t	bits_per_sample;
   uint32_t	bytes_per_frame;
@@ -207,13 +207,13 @@ static int write_buffer_16 (jack_driver_t *this, unsigned char *data, int len)
  * If there is not enough data in the buffer remaining parts will be filled
  * with silence.
  */
-static int read_buffer (jack_driver_t *this, float **bufs, int cnt,
-			int num_bufs, float gain)
+static jack_nframes_t read_buffer (jack_driver_t *this, float **bufs, jack_nframes_t cnt,
+                                   unsigned num_bufs, float gain)
 {
   int buffered = buf_used (this);
-  int i, j;
-  int orig_cnt = cnt;
-  if (cnt * sizeof (float) * num_bufs > buffered)
+  unsigned j;
+  jack_nframes_t i, orig_cnt = cnt;
+  if (cnt * num_bufs > buffered / sizeof(float))
     cnt = buffered / (sizeof (float) * num_bufs);
 
   uint32_t read_pos = this->read_pos;
@@ -281,14 +281,14 @@ static int jack_callback (jack_nframes_t nframes, void *arg)
   }
 
   float *bufs[MAX_CHANS];
-  int i;
+  unsigned i;
   for (i = 0; i < this->num_channels; i++)
     bufs[i] = jack_port_get_buffer (this->ports[i], nframes);
 
   if (this->paused || this->underrun) {
     silence (bufs, nframes, this->num_channels);
   } else {
-    int frames_read = read_buffer (this, bufs, nframes, this->num_channels, gain);
+    jack_nframes_t frames_read = read_buffer (this, bufs, nframes, this->num_channels, gain);
     if (frames_read < nframes) {
       xprintf (this->xine, XINE_VERBOSITY_LOG,
 	       "jack_callback: underrun - frames read: %d\n", frames_read);
@@ -312,10 +312,9 @@ static void jack_shutdown (void *arg)
  * Return 1 on success, 0 on failure
  * All error handling rests with the caller, we just try to open the device here
  */
-static int jack_open_device (ao_driver_t *this_gen, char *jack_device,
-			     int32_t *poutput_sample_rate, int num_channels)
+static int jack_open_device (jack_driver_t *this, const char *jack_device,
+                             uint32_t *poutput_sample_rate, int num_channels)
 {
-  jack_driver_t *this = (jack_driver_t *) this_gen;
   const char **matching_ports = NULL;
   jack_client_t *client = this->client;
 
@@ -425,16 +424,9 @@ static void ao_jack_close (ao_driver_t *this_gen)
 /*
  * open the audio device for writing to
  */
-static int ao_jack_open (ao_driver_t *this_gen, uint32_t bits, uint32_t rate,
-			 int mode)
+static int ao_jack_open_int (jack_driver_t *this, const char *jack_device,
+                             uint32_t bits, uint32_t rate, int mode)
 {
-  jack_driver_t *this = (jack_driver_t *) this_gen;
-  config_values_t *config = this->xine->config;
-  char *jack_device;
-
-  jack_device =
-    config->lookup_entry (config, "audio.device.jack_device_name")->str_value;
-
   xprintf (this->xine, XINE_VERBOSITY_DEBUG,
 	   "ao_jack_open: ao_open rate=%d, mode=%d, bits=%d dev=%s\n", rate,
 	   mode, bits, jack_device);
@@ -459,7 +451,7 @@ static int ao_jack_open (ao_driver_t *this_gen, uint32_t bits, uint32_t rate,
       return this->output_sample_rate;
     }
 
-    ao_jack_close (this_gen);
+    ao_jack_close (&this->ao_driver);
   }
 
 
@@ -510,7 +502,7 @@ static int ao_jack_open (ao_driver_t *this_gen, uint32_t bits, uint32_t rate,
   /*
    * open audio device
    */
-  if (!jack_open_device (this_gen, jack_device, &(this->output_sample_rate),
+  if (!jack_open_device (this, jack_device, &(this->output_sample_rate),
 			 this->num_channels))
     return 0;
 
@@ -523,6 +515,24 @@ static int ao_jack_open (ao_driver_t *this_gen, uint32_t bits, uint32_t rate,
   return this->output_sample_rate;
 }
 
+static int ao_jack_open (ao_driver_t *this_gen,
+                         uint32_t bits, uint32_t rate, int mode)
+{
+  jack_driver_t *this = (jack_driver_t *) this_gen;
+  char *jack_device;
+  int result;
+
+  jack_device = xine_config_lookup_string(this->xine, "audio.device.jack_device_name");
+  if (!jack_device) {
+    xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+             "ao_jack_open: no audio.device.jack_device_name in config\n");
+    /* not fatal - this is tested later */
+  }
+
+  result = ao_jack_open_int(this, jack_device, bits, rate, mode);
+  xine_config_free_string(this->xine, &jack_device);
+  return result;
+}
 
 static int ao_jack_num_channels (ao_driver_t *this_gen)
 {
@@ -591,7 +601,7 @@ static int ao_jack_write (ao_driver_t *this_gen, int16_t *frame_buffer,
     /* Sleep to save CPU */
     int until_callback =
       this->fragment_size - jack_frames_since_cycle_start (this->client);
-    if ((until_callback < 0) || (until_callback > this->fragment_size)) {
+    if ((until_callback < 0) || ((unsigned)until_callback > this->fragment_size)) {
       xprintf (this->xine, XINE_VERBOSITY_DEBUG,
 	       "ao_jack_write: Invalid until_callback %d\n", until_callback);
       until_callback = this->fragment_size;
@@ -725,6 +735,8 @@ static ao_driver_t *open_jack_plugin (audio_driver_class_t *class_gen,
   }
 
   this = calloc(1, sizeof (jack_driver_t));
+  if (!this)
+    goto err_out;
 
   rate = jack_get_sample_rate (client);
   xprintf (class->xine, XINE_VERBOSITY_DEBUG,
