@@ -1,7 +1,7 @@
 /* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*- */
 
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -143,6 +143,8 @@ static void __xine_pa_stream_request_callback(pa_stream *s, size_t nbytes, void
 {
   pulse_driver_t * this = (pulse_driver_t*) this_gen;
 
+  (void)s;
+  (void)nbytes;
   pa_threaded_mainloop_signal(this->mainloop, 0);
 }
 
@@ -156,6 +158,7 @@ static void __xine_pa_stream_notify_callback(pa_stream *s, void *this_gen)
 {
   pulse_driver_t * this = (pulse_driver_t*) this_gen;
 
+  (void)s;
   pa_threaded_mainloop_signal(this->mainloop, 0);
 }
 
@@ -170,6 +173,7 @@ static void __xine_pa_stream_success_callback(pa_stream *s, int success, void *t
 {
   pulse_driver_t * this = (pulse_driver_t*) this_gen;
 
+  (void)s;
   if (!success)
     xprintf (this->xine, XINE_VERBOSITY_DEBUG, "audio_pulse_out: stream operation failed: %s\n", pa_strerror(pa_context_errno(this->context)));
 
@@ -187,6 +191,7 @@ static void __xine_pa_context_success_callback(pa_context *c, int success, void
 {
   pulse_driver_t *this = (pulse_driver_t*) this_gen;
 
+  (void)c;
   if (!success)
     xprintf (this->xine, XINE_VERBOSITY_DEBUG, "audio_pulse_out: context operation failed: %s\n", pa_strerror(pa_context_errno(this->context)));
 
@@ -210,6 +215,7 @@ static void __xine_pa_sink_info_callback(pa_context *c, const pa_sink_input_info
 
   pulse_driver_t *const this = (pulse_driver_t *) userdata;
 
+  (void)c;
   if (is_last < 0) {
     xprintf (this->xine, XINE_VERBOSITY_DEBUG, "audio_pulse_out: Failed to get sink input info: %s\n",
              pa_strerror(pa_context_errno(this->context)));
@@ -244,9 +250,8 @@ static void __xine_pa_sink_info_callback(pa_context *c, const pa_sink_input_info
   event.data_length = sizeof(data);
 
   pthread_mutex_lock(&this->xine->streams_lock);
-  for(ite = xine_list_front(this->xine->streams); ite; ite =
-    xine_list_next(this->xine->streams, ite)) {
-    stream = xine_list_get_value(this->xine->streams, ite);
+  ite = NULL;
+  while ((stream = xine_list_next_value (this->xine->streams, &ite))) {
     event.stream = stream;
     xine_event_send(stream, &event);
   }
@@ -265,8 +270,9 @@ static void __xine_pa_context_subscribe_callback(pa_context *c,
     pa_subscription_event_type_t t, uint32_t idx, void *this_gen)
 {
   pulse_driver_t * this = (pulse_driver_t*) this_gen;
-  int index;
+  uint32_t index;
 
+  (void)c;
   if (this->stream == NULL)
     return;
 
@@ -627,6 +633,7 @@ static int ao_pulse_bytes_per_frame(ao_driver_t *this_gen)
 
 static int ao_pulse_get_gap_tolerance (ao_driver_t *this_gen)
 {
+  (void)this_gen;
   return GAP_TOLERANCE;
 }
 
@@ -943,6 +950,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
 
   lprintf ("audio_pulse_out: open_plugin called\n");
 
+  (void)data;
   this = calloc(1, sizeof (pulse_driver_t));
   if (!this)
     return NULL;
@@ -1067,12 +1075,13 @@ static void dispose_class (audio_driver_class_t *this_gen) {
   free(this);
 }
 
-static void *init_class (xine_t *xine, void *data) {
+static void *init_class (xine_t *xine, const void *data) {
 
   pulse_class_t        *this;
 
   lprintf ("audio_pulse_out: init class\n");
 
+  (void)data;
   this = calloc(1, sizeof (pulse_class_t));
   if (!this)
     return NULL;
@@ -1087,7 +1096,7 @@ static void *init_class (xine_t *xine, void *data) {
 }
 
 static const ao_info_t ao_info_pulse = {
-  12
+  .priority = 12,
 };
 
 /*
@@ -1097,5 +1106,7 @@ static const ao_info_t ao_info_pulse = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_AUDIO_OUT, 9, "pulseaudio", XINE_VERSION_CODE, &ao_info_pulse, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
+
+
