@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2011 the xine project
+ * Copyright (C) 2001-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -1011,7 +1011,7 @@ static ao_driver_t *ao_sun_open_plugin (audio_driver_class_t *class_gen, const v
 /*
  * class functions
  */
-static void *ao_sun_init_class (xine_t *xine, void *data) {
+static void *ao_sun_init_class (xine_t *xine, const void *data) {
   sun_class_t         *this;
 
   this = calloc(1, sizeof (sun_class_t));
@@ -1030,7 +1030,7 @@ static void *ao_sun_init_class (xine_t *xine, void *data) {
 
 
 static const ao_info_t ao_info_sun = {
-  10
+  .priority = 10,
 };
 
 /*
@@ -1040,5 +1040,5 @@ static const ao_info_t ao_info_sun = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_AUDIO_OUT, AO_SUN_IFACE_VERSION, "sun", XINE_VERSION_CODE, &ao_info_sun, ao_sun_init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
