@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2018 the xine project
+ * Copyright (C) 2001-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -21,41 +21,35 @@
 #ifndef HAVE_XINE_DECODER_H
 #define HAVE_XINE_DECODER_H
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <sys/types.h>
 
-#ifdef HAVE_FFMPEG_AVUTIL_H
-#  include <avcodec.h>
-#else
-#  include <libavcodec/avcodec.h>
-#  include <libavutil/mem.h>
-#endif
+#include <xine.h>
+
+#if defined LIBAVCODEC_VERSION_INT
 
 typedef struct ff_codec_s {
   uint32_t          type;
-#if defined LIBAVCODEC_VERSION_INT && LIBAVCODEC_VERSION_INT >= ((54<<16)|(25<<8))
+# if LIBAVCODEC_VERSION_INT >= ((54<<16)|(25<<8))
   enum AVCodecID    id;
-#else
+# else
   enum CodecID      id;
-#endif
+# endif
   const char       *name;
 } ff_codec_t;
 
+extern const ff_codec_t ff_audio_lookup[];
+extern const ff_codec_t ff_video_lookup[];
+extern const size_t ff_video_lookup_entries;
+extern const size_t ff_audio_lookup_entries;
+
+#endif
+
 void *init_audio_plugin (xine_t *xine, const void *data);
 void *init_video_plugin (xine_t *xine, const void *data);
 void *init_avio_input_plugin (xine_t *xine, const void *data);
 void *init_avformat_input_plugin (xine_t *xine, const void *data);
 void *init_avformat_demux_plugin (xine_t *xine, const void *data);
 
-extern const decoder_info_t dec_info_ffmpeg_video;
-extern const decoder_info_t dec_info_ffmpeg_wmv8;
-extern const decoder_info_t dec_info_ffmpeg_wmv9;
-extern const decoder_info_t dec_info_ffmpeg_audio;
-extern const demuxer_info_t demux_info_avformat;
-extern const input_info_t   input_info_avformat;
-extern const input_info_t   input_info_avio;
-
 /* communication between avio/avformat input and avformat demux plugins */
 #define INPUT_OPTIONAL_DATA_pb         0x1000
 #define INPUT_OPTIONAL_DATA_fmt_ctx    0x1001
@@ -64,7 +58,6 @@ extern const input_info_t   input_info_avio;
 #define INPUT_AVIO_ID     "avio"
 #define DEMUX_AVFORMAT_ID "avformat"
 
-extern pthread_once_t once_control;
 void init_once_routine(void);
 
 extern pthread_mutex_t ffmpeg_lock;
