@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2017 the xine project
+ * Copyright (C) 2001-2022 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -69,203 +69,194 @@ typedef struct {
   int                  sample_rate;
   int                  frame_size;
   int                  running_time;
-
-  off_t                data_start;
+  unsigned             frame_number;
 
   uint32_t             buf_type;
 
+  unsigned             start_offset;
 } demux_ac3_t;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_ac3_class_t;
-
-/* borrow some knowledge from the AC3 decoder */
-struct frmsize_s {
-  uint16_t bit_rate;
-  uint16_t frm_size[3];
-};
-
-static const struct frmsize_s frmsizecod_tbl[64] =
+static int _frame_size(const uint8_t *buf)
 {
-  { 32  ,{64   ,69   ,96   } },
-  { 32  ,{64   ,70   ,96   } },
-  { 40  ,{80   ,87   ,120  } },
-  { 40  ,{80   ,88   ,120  } },
-  { 48  ,{96   ,104  ,144  } },
-  { 48  ,{96   ,105  ,144  } },
-  { 56  ,{112  ,121  ,168  } },
-  { 56  ,{112  ,122  ,168  } },
-  { 64  ,{128  ,139  ,192  } },
-  { 64  ,{128  ,140  ,192  } },
-  { 80  ,{160  ,174  ,240  } },
-  { 80  ,{160  ,175  ,240  } },
-  { 96  ,{192  ,208  ,288  } },
-  { 96  ,{192  ,209  ,288  } },
-  { 112 ,{224  ,243  ,336  } },
-  { 112 ,{224  ,244  ,336  } },
-  { 128 ,{256  ,278  ,384  } },
-  { 128 ,{256  ,279  ,384  } },
-  { 160 ,{320  ,348  ,480  } },
-  { 160 ,{320  ,349  ,480  } },
-  { 192 ,{384  ,417  ,576  } },
-  { 192 ,{384  ,418  ,576  } },
-  { 224 ,{448  ,487  ,672  } },
-  { 224 ,{448  ,488  ,672  } },
-  { 256 ,{512  ,557  ,768  } },
-  { 256 ,{512  ,558  ,768  } },
-  { 320 ,{640  ,696  ,960  } },
-  { 320 ,{640  ,697  ,960  } },
-  { 384 ,{768  ,835  ,1152 } },
-  { 384 ,{768  ,836  ,1152 } },
-  { 448 ,{896  ,975  ,1344 } },
-  { 448 ,{896  ,976  ,1344 } },
-  { 512 ,{1024 ,1114 ,1536 } },
-  { 512 ,{1024 ,1115 ,1536 } },
-  { 576 ,{1152 ,1253 ,1728 } },
-  { 576 ,{1152 ,1254 ,1728 } },
-  { 640 ,{1280 ,1393 ,1920 } },
-  { 640 ,{1280 ,1394 ,1920 } }
-};
+  static const uint8_t byterates[19] = {
+    4,   5,   6,   7,   8,  10,  12,  14,
+    16,  20,  24,  28,  32,  40,  48,  56,
+    64,  72,  80
+  };
+  uint32_t idx, rate;
+  idx = buf[4];
+  if ((idx & 0x3f) > 37)
+    return -1;
+  rate = (uint32_t)byterates[(idx >> 1) & 0x1f];
+  if (idx & 0x80) {
+    if (idx & 0x40) {
+      return -1;
+    } else {
+      return rate * 24 * 2;
+    }
+  } else {
+    if (idx & 0x40) {
+      return ((rate * 16 * 48000 / 44100) + (idx & 0x01)) * 2;
+    } else {
+      return rate * 16 * 2;
+    }
+  }
+  return -1;
+}
 
 /* returns 1 if the AC3 file was opened successfully, 0 otherwise */
 static int open_ac3_file(demux_ac3_t *this) {
-  int i;
-  int offset = 0;
-  size_t peak_size = 0;
-  int spdif_mode = 0;
-  uint32_t syncword = 0;
-  uint32_t blocksize;
-  uint8_t *peak;
+  buf_element_t *buf = NULL;
+  uint32_t offset = 0;
+  int ret = 0;
 
-  blocksize = this->input->get_blocksize(this->input);
-  if (blocksize && INPUT_IS_SEEKABLE(this->input)) {
-    this->input->seek(this->input, 0, SEEK_SET);
-    buf_element_t *buf = this->input->read_block(this->input,
-						 this->stream->audio_fifo,
-						 blocksize);
-    this->input->seek(this->input, 0, SEEK_SET);
+  do {
+    uint32_t bsize = 0;
+    int spdif_mode = 0;
+    uint8_t hb[MAX_PREVIEW_SIZE];
+    const uint8_t *b;
 
-    if (!buf)
-      return 0;
-
-    peak = alloca(peak_size = buf->size);
-    xine_fast_memcpy(peak, buf->content, peak_size);
-
-    buf->free_buffer(buf);
-  } else {
-    peak = alloca(peak_size = MAX_PREVIEW_SIZE);
-
-    if (_x_demux_read_header(this->input, peak, peak_size) != peak_size)
-      return 0;
-  }
-
-  lprintf("peak size: %zu\n", peak_size);
-
-  /* Check for wav header, as we'll handle AC3 with a wav header shoved
-  * on the front for CD burning */
-  /* FIXME: This is risky. Real LPCM may contain anything, even sync words. */
-  if ( memcmp(peak, "RIFF", 4) == 0 || memcmp(&peak[8], "WAVEfmt ", 8) == 0 ) {
-    /* Check this looks like a cd audio wav */
-    unsigned int audio_type;
-    xine_waveformatex *wave = (xine_waveformatex *) &peak[20];
-
-    _x_waveformatex_le2me(wave);
-    audio_type = _x_formattag_to_buf_audio(wave->wFormatTag);
-
-    if ((audio_type != BUF_AUDIO_LPCM_LE) || (wave->nChannels != 2) ||
-         (wave->nSamplesPerSec != 44100) || (wave->wBitsPerSample != 16))
-      return 0;
-
-    lprintf("looks like a cd audio wav file\n");
-
-    /* Find the data chunk */
-    offset = 20 + _X_LE_32(&peak[16]);
-    while (offset < peak_size-8) {
-      unsigned int chunk_tag = _X_LE_32(&peak[offset]);
-      unsigned int chunk_size = _X_LE_32(&peak[offset+4]);
-
-      if (chunk_tag == DATA_TAG) {
-        offset += 8;
-        lprintf("found the start of the data at offset %d\n", offset);
+    do {
+      if (!INPUT_IS_SEEKABLE (this->input))
         break;
-      } else
-        offset += 8 + chunk_size;
+      bsize = this->input->get_blocksize (this->input);
+      if (!bsize)
+        break;
+      if (this->input->seek (this->input, 0, SEEK_SET) != 0)
+        break;
+      buf = this->input->read_block (this->input, this->stream->audio_fifo, bsize);
+      if (!buf)
+        break;
+      if (this->input->seek (this->input, 0, SEEK_SET) != 0) { /* should not happen */
+        buf->free_buffer (buf);
+        buf = NULL;
+        break;
+      }
+      b = buf->content;
+      bsize = buf->size;
+    } while (0);
+    if (!buf) {
+      int s;
+      s = _x_demux_read_header (this->input, hb, MAX_PREVIEW_SIZE);
+      if (s <= 0)
+        break;
+      bsize = s;
+      b = hb;
     }
-  }
+    lprintf ("peek size: %u\n", (unsigned int)bsize);
+    if (bsize < 16)
+      break;
 
-  /* Look for a valid AC3 sync word */
-  for (i=offset; i<peak_size; i++) {
-    if ((syncword & 0xffff) == 0x0b77) {
-      this->data_start = i-2;
-      lprintf("found AC3 syncword at offset %d\n", i-2);
+    do {
+      /* Check for wav header, as we'll handle AC3 with a wav header shoved
+       * on the front for CD burning.
+       * FIXME: This is risky. Real LPCM may contain anything, even sync words. */
+      unsigned int audio_type;
+      xine_waveformatex *wave;
+      uint32_t o;
+
+      /* Check this looks like a cd audio wav */
+      if (bsize < 20 + sizeof (xine_waveformatex))
+        break;
+      if (memcmp (b, "RIFF", 4) && memcmp (b + 8, "WAVEfmt ", 8))
+        break;
+      wave = (xine_waveformatex *)(b + 20);
+      _x_waveformatex_le2me (wave);
+      audio_type = _x_formattag_to_buf_audio (wave->wFormatTag);
+      if ((audio_type != BUF_AUDIO_LPCM_LE) || (wave->nChannels != 2) ||
+          (wave->nSamplesPerSec != 44100) || (wave->wBitsPerSample != 16))
+        break;
+      lprintf ("looks like a cd audio wav file\n");
+
+      /* Find the data chunk */
+      o = _X_LE_32 (b + 16);
+      if (o > bsize - 20 - 8)
+        break;
+      o += 20;
+      do {
+        uint32_t chunk_tag = _X_LE_32 (b + o);
+        uint32_t chunk_size = _X_LE_32 (b + o + 4);
+        o += 8;
+        if (chunk_tag == DATA_TAG) {
+          offset = o;
+          lprintf ("found the start of the data at offset %d\n", offset);
+          break;
+        }
+        if (o >= bsize)
+          break;
+        if (chunk_size > bsize - o)
+          break;
+        o += chunk_size;
+      } while (o <= bsize - 8);
+    } while (0);
+
+    /* Look for a valid AC3 sync word */
+    {
+      uint32_t syncword = 0;
+      while (offset < bsize) {
+        if ((syncword & 0xffff) == 0x0b77) {
+          offset -= 2;
+          lprintf ("found AC3 syncword at offset %u\n", (unsigned int)offset);
+          break;
+        }
+        if ((syncword == 0x72f81f4e) && (b[offset] == 0x01)) {
+          spdif_mode = 1;
+          lprintf ("found AC3 SPDIF header at offset %u\n", (unsigned int)offset - 4);
+          offset += 4;
+          break;
+        }
+        syncword = (syncword << 8) | b[offset];
+        offset++;
+      }
+    }
+    if (offset >= bsize - 2)
+      break;
+
+    this->start_offset = offset;
+
+    if (spdif_mode) {
+      this->sample_rate = 44100;
+      this->frame_size = 256 * 6 * 4;
+      this->buf_type = BUF_AUDIO_DNET;
+      ret = 1;
       break;
     }
 
-    if ((syncword == 0x72f81f4e) && (peak[i] == 0x01)) {
-      spdif_mode = 1;
-      this->data_start = i+4;
-      lprintf("found AC3 SPDIF header at offset %d\n", i-4);
+    this->frame_size = _frame_size(b + offset);
+    if (this->frame_size < 0)
       break;
-    }
-
-    syncword = (syncword << 8) | peak[i];
-  }
-
-  if (i >= peak_size-2)
-    return 0;
-
-  if (spdif_mode) {
-    this->sample_rate = 44100;
-    this->frame_size = 256*6*4;
-    this->buf_type = BUF_AUDIO_DNET;
-  } else {
-    int fscod, frmsizecod;
-
-    fscod = peak[this->data_start+4] >> 6;
-    frmsizecod = peak[this->data_start+4] & 0x3F;
-
-    if ((fscod > 2) || (frmsizecod > 37))
-      return 0;
-
-    this->frame_size = frmsizecod_tbl[frmsizecod].frm_size[fscod] * 2;
-
-    /* convert the sample rate to a more useful number */
-    switch (fscod) {
-      case 0:
-        this->sample_rate = 48000;
-        break;
-      case 1:
-        this->sample_rate = 44100;
-        break;
-      default:
-        this->sample_rate = 32000;
+    {
+      const uint16_t srate[4] = { 48000, 44100, 32000, 0 };
+      this->sample_rate = srate[b[offset + 4] >> 6];
+      if (this->sample_rate == 0)
         break;
     }
 
     /* Look for a second sync word */
-    if ((this->data_start+this->frame_size+1 >= peak_size) ||
-        (peak[this->data_start+this->frame_size] != 0x0b) ||
-        (peak[this->data_start+this->frame_size + 1] != 0x77)) {
-      return 0;
+    if ((offset + this->frame_size + 1 >= bsize) ||
+        (b[offset + this->frame_size] != 0x0b) ||
+        (b[offset + this->frame_size + 1] != 0x77)) {
+      break;
     }
-
-    lprintf("found second AC3 sync word\n");
-
+    lprintf ("found second AC3 sync word\n");
     this->buf_type = BUF_AUDIO_A52;
+    ret = 1;
+  } while (0);
+
+  if (buf)
+    buf->free_buffer (buf);
+
+  if (ret) {
+    this->running_time = this->input->get_length (this->input) - offset;
+    this->running_time /= this->frame_size;
+    this->running_time *= (90000 / 1000) * (256 * 6);
+    this->running_time /= this->sample_rate;
+
+    lprintf ("sample rate: %d\n", this->sample_rate);
+    lprintf ("frame size: %d\n", this->frame_size);
+    lprintf("running time: %d\n", this->running_time);
   }
-
-  this->running_time = this->input->get_length(this->input) -
-                       this->data_start;
-  this->running_time /= this->frame_size;
-  this->running_time *= (90000 / 1000) * (256 * 6);
-  this->running_time /= this->sample_rate;
-
-  lprintf("sample rate: %d\n", this->sample_rate);
-  lprintf("frame size: %d\n", this->frame_size);
-  lprintf("running time: %d\n", this->running_time);
-
-  return 1;
+  return ret;
 }
 
 static int demux_ac3_send_chunk (demux_plugin_t *this_gen) {
@@ -274,11 +265,12 @@ static int demux_ac3_send_chunk (demux_plugin_t *this_gen) {
   buf_element_t *buf = NULL;
   off_t current_stream_pos;
   int64_t audio_pts;
-  int frame_number;
   uint32_t blocksize;
 
   current_stream_pos = this->input->get_current_pos(this->input);
-  frame_number = current_stream_pos / this->frame_size;
+  if (this->seek_flag) {
+    this->frame_number = current_stream_pos / this->frame_size;
+  }
 
   /*
    * Each frame represents 256*6 new audio samples according to the a52 spec.
@@ -288,10 +280,11 @@ static int demux_ac3_send_chunk (demux_plugin_t *this_gen) {
    *                     -------------  *  -----------  *  ---------
    *                        1 frame        sample rate       1 sec
    */
-  audio_pts = frame_number;
+  audio_pts = this->frame_number;
   audio_pts *= 90000;
   audio_pts *= 256 * 6;
   audio_pts /= this->sample_rate;
+  this->frame_number++;
 
   if (this->seek_flag) {
     _x_demux_control_newpts(this->stream, audio_pts, BUF_FLAG_SEEK);
@@ -308,8 +301,35 @@ static int demux_ac3_send_chunk (demux_plugin_t *this_gen) {
     }
   } else {
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->size = this->input->read(this->input, buf->content,
-                                  this->frame_size);
+    _x_assert(buf->max_size >= this->frame_size);
+
+    if (this->buf_type != BUF_AUDIO_A52) {
+      buf->size = this->input->read(this->input, buf->content, this->frame_size);
+    } else {
+
+      buf->size = this->input->read(this->input, buf->content, 8);
+      if (buf->size == 8) {
+
+        /* re-sync (rare case, usually after seek) */
+        while (buf->content[0] != 0x0b || buf->content[1] != 0x77) {
+          memmove(buf->content, buf->content + 1, buf->size);
+          if (this->input->read(this->input, buf->content + buf->size - 1, 1) != 1) {
+            buf->size -= 1;
+            break;
+          }
+        }
+
+        /* check frame size and read the rest */
+        if (buf->content[0] == 0x0b && buf->content[1] == 0x77) {
+          int got, frame_size = _frame_size(buf->content);
+          if (frame_size > 0)
+            this->frame_size = frame_size;
+          got = this->input->read(this->input, buf->content + buf->size, this->frame_size - buf->size);
+          if (got > 0)
+            buf->size += got;
+        }
+      }
+    }
   }
 
   if (buf->size <= 0) {
@@ -364,6 +384,9 @@ static int demux_ac3_seek (demux_plugin_t *this_gen,
   start_pos = (off_t) ( (double) start_pos / 65535 *
               this->input->get_length (this->input) );
 
+  (void)start_time;
+  (void)playing;
+
   this->seek_flag = 1;
   this->status = DEMUX_OK;
   _x_demux_flush_engine (this->stream);
@@ -377,6 +400,10 @@ static int demux_ac3_seek (demux_plugin_t *this_gen,
    * multiply by the frame alignment to determine the new starting block */
   start_pos /= this->frame_size;
   start_pos *= this->frame_size;
+
+  /* skip possible file header */
+  start_pos += this->start_offset;
+
   this->input->seek(this->input, start_pos, SEEK_SET);
 
   return this->status;
@@ -396,11 +423,15 @@ static int demux_ac3_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_ac3_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_ac3_get_optional_data(demux_plugin_t *this_gen,
                                        void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -409,7 +440,10 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_ac3_t   *this;
 
-  this         = calloc(1, sizeof(demux_ac3_t));
+  this = calloc(1, sizeof(demux_ac3_t));
+  if (!this)
+    return NULL;
+
   this->stream = stream;
   this->input  = input;
 
@@ -446,17 +480,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-void *demux_ac3_init_plugin (xine_t *xine, void *data) {
-  demux_ac3_class_t     *this;
+void *demux_ac3_init_plugin (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_ac3_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("Raw AC3 demux plugin");
-  this->demux_class.identifier      = "AC3";
-  this->demux_class.mimetypes       = "audio/ac3: ac3: Dolby Digital audio;";
-  this->demux_class.extensions      = "ac3";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_ac3_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("Raw AC3 demux plugin"),
+    .identifier      = "AC3",
+    .mimetypes       = "audio/ac3: ac3: Dolby Digital audio;",
+    .extensions      = "ac3",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_ac3_class;
 }
