@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2009 the xine project
+ * Copyright (C) 2001-2022 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -54,7 +54,6 @@ typedef struct {
   demux_plugin_t       demux_plugin;
 
   xine_stream_t       *stream;
-  fifo_buffer_t       *video_fifo;
   fifo_buffer_t       *audio_fifo;
   input_plugin_t      *input;
   int                  status;
@@ -65,14 +64,13 @@ typedef struct {
 
   off_t                data_start;
   off_t                data_size;
+  int                  seek_align;
+  int                  chunk_size;
 
   int                  send_newpts;
   int                  seek_flag;  /* this is set when a seek just occurred */
 } demux_wav_t;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_wav_class_t;
 
 static int demux_wav_get_stream_length (demux_plugin_t *this_gen);
 
@@ -87,7 +85,8 @@ static int find_chunk_by_tag(demux_wav_t *this, const uint32_t given_chunk_tag,
   uint8_t chunk_preamble[8];
 
   /* search for the chunks from the start of the WAV file */
-  this->input->seek(this->input, WAV_SIGNATURE_SIZE, SEEK_SET);
+  if (this->input->seek(this->input, WAV_SIGNATURE_SIZE, SEEK_SET) != WAV_SIGNATURE_SIZE)
+    return 0;
 
   while (1) {
     if (this->input->read(this->input, chunk_preamble, 8) != 8) {
@@ -104,36 +103,45 @@ static int find_chunk_by_tag(demux_wav_t *this, const uint32_t given_chunk_tag,
         *found_chunk_pos = this->input->get_current_pos(this->input);
       return 1;
     } else {
-      this->input->seek(this->input, chunk_size, SEEK_CUR);
+      if (this->input->seek(this->input, chunk_size, SEEK_CUR) < 0)
+        return 0;
     }
   }
 }
 
-/* returns 1 if the WAV file was opened successfully, 0 otherwise */
-static int open_wav_file(demux_wav_t *this) {
+static int probe_wav_file(input_plugin_t *input) {
   uint8_t signature[WAV_SIGNATURE_SIZE];
-  off_t wave_pos;
-  uint32_t wave_size;
 
   /* check the signature */
-  if (_x_demux_read_header(this->input, signature, WAV_SIGNATURE_SIZE) != WAV_SIGNATURE_SIZE)
+  if (_x_demux_read_header(input, signature, WAV_SIGNATURE_SIZE) != WAV_SIGNATURE_SIZE)
     return 0;
 
   if (memcmp(signature, "RIFF", 4) || memcmp(&signature[8], "WAVE", 4) )
     return 0;
 
+  return 1;
+}
+
+/* returns 1 if the WAV file was opened successfully, 0 otherwise */
+static int open_wav_file(demux_wav_t *this) {
+  off_t wave_pos;
+  uint32_t wave_size;
+
   /* search for the 'fmt ' chunk first */
   wave_pos = 0;
   if (find_chunk_by_tag(this, fmt_TAG, &wave_size, &wave_pos)==0)
     return 0;
   this->wave_size = wave_size;
 
-  this->input->seek(this->input, wave_pos, SEEK_SET);
-  this->wave = malloc( this->wave_size );
+  if (this->input->seek(this->input, wave_pos, SEEK_SET) != wave_pos)
+    return 0;
 
-  if (!this->wave || this->input->read(this->input, (void *)this->wave, this->wave_size) !=
+  this->wave = malloc( this->wave_size );
+  if (!this->wave)
+    return 0;
+
+  if (this->input->read(this->input, (void *)this->wave, this->wave_size) !=
     this->wave_size) {
-    free (this->wave);
     return 0;
   }
   _x_waveformatex_le2me(this->wave);
@@ -143,25 +151,27 @@ static int open_wav_file(demux_wav_t *this) {
   }
 
   if (this->wave->nChannels <= 0) {
-    free (this->wave);
     return 0;
   }
 
   /* search for the 'data' chunk */
   this->data_start = this->data_size = 0;
-  if (find_chunk_by_tag(this, data_TAG, NULL, &this->data_start)==0)
-  {
-    free (this->wave);
+  if (find_chunk_by_tag (this, data_TAG, NULL, &this->data_start) == 0)
     return 0;
-  }
-  else
-  {
-    /* Get the data length from the file itself, instead of the data
-     * TAG, for broken files */
-    this->input->seek(this->input, this->data_start, SEEK_SET);
-    this->data_size = this->input->get_length(this->input);
-    return 1;
-  }
+
+  /* Get the data length from the file itself, instead of the data TAG, for broken files */
+  if (this->input->seek (this->input, this->data_start, SEEK_SET) != this->data_start)
+    return 0;
+
+  /* XXX */
+  if (this->wave->nBlockAlign <= 0)
+    this->wave->nBlockAlign = 4;
+  this->seek_align = this->wave->nBlockAlign;
+  this->chunk_size = (this->wave->nBlockAlign < PREFERED_BLOCK_SIZE)
+                   ? PREFERED_BLOCK_SIZE / this->wave->nBlockAlign * this->wave->nBlockAlign
+                   : this->wave->nBlockAlign;
+  this->data_size = this->input->get_length (this->input);
+  return 1;
 }
 
 static int demux_wav_send_chunk(demux_plugin_t *this_gen) {
@@ -174,14 +184,9 @@ static int demux_wav_send_chunk(demux_plugin_t *this_gen) {
 
   /* just load data chunks from wherever the stream happens to be
    * pointing; issue a DEMUX_FINISHED status if EOF is reached */
-  if (this->wave->nBlockAlign < PREFERED_BLOCK_SIZE) {
-    remaining_sample_bytes = PREFERED_BLOCK_SIZE / this->wave->nBlockAlign *
-                             this->wave->nBlockAlign;
-  } else{
-    remaining_sample_bytes = this->wave->nBlockAlign;
-  }
-  current_file_pos =
-    this->input->get_current_pos(this->input) - this->data_start;
+  remaining_sample_bytes = this->chunk_size;
+
+  current_file_pos = this->input->get_current_pos(this->input) - this->data_start;
 
   current_pts = current_file_pos;
   current_pts *= 90000;
@@ -199,12 +204,21 @@ static int demux_wav_send_chunk(demux_plugin_t *this_gen) {
     }
 
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    if( this->data_size )
-      buf->extra_info->input_normpos = (int)( (double) current_file_pos * 65535 / this->data_size);
+    do {
+      if (this->data_size <= 0)
+        break;
+      /* file being written right now? */
+      if (current_file_pos > this->data_size) {
+        this->data_size = this->input->get_length (this->input);
+        if (this->data_size <= 0)
+          break;
+      }
+      buf->extra_info->input_normpos = (int)((double)current_file_pos * 65535 / this->data_size);
+    } while (0);
     buf->extra_info->input_time = current_pts / 90;
     buf->pts = current_pts;
 
-    if (remaining_sample_bytes > buf->max_size)
+    if ((int)remaining_sample_bytes > buf->max_size)
       buf->size = buf->max_size;
     else
       buf->size = remaining_sample_bytes;
@@ -250,7 +264,6 @@ static void demux_wav_send_headers(demux_plugin_t *this_gen) {
   demux_wav_t *this = (demux_wav_t *) this_gen;
   buf_element_t *buf;
 
-  this->video_fifo  = this->stream->video_fifo;
   this->audio_fifo  = this->stream->audio_fifo;
 
   this->status = DEMUX_OK;
@@ -287,21 +300,21 @@ static int demux_wav_seek (demux_plugin_t *this_gen,
                            off_t start_pos, int start_time, int playing) {
 
   demux_wav_t *this = (demux_wav_t *) this_gen;
-  start_pos = (off_t) ( (double) start_pos / 65535 *
-              this->data_size );
+  start_pos = this->data_size > 0 ? (off_t)((double)start_pos * this->data_size / 65535) : 0;
 
   this->status = DEMUX_OK;
 
-  this->send_newpts = 1;
-  if (playing) {
-    this->seek_flag = 1;
-    _x_demux_flush_engine (this->stream);
-  }
   /* if input is non-seekable, do not proceed with the rest of this
    * seek function */
-  if (!INPUT_IS_SEEKABLE(this->input))
+  if (!(this->input->get_capabilities (this->input) & (INPUT_CAP_SEEKABLE | INPUT_CAP_SLOW_SEEKABLE)))
     return this->status;
 
+  this->send_newpts = 1;
+  if (playing) {
+    this->seek_flag = BUF_FLAG_SEEK;
+    _x_demux_flush_engine (this->stream);
+  }
+
   /* time-based seeking, the start_pos code will then align the blocks
    * if necessary */
   if (start_time != 0) {
@@ -323,8 +336,8 @@ static int demux_wav_seek (demux_plugin_t *this_gen,
      * the block alignment integer-wise, and multiply the quotient by the
      * block alignment to get the new aligned offset. Add the data start
      * offset and seek to the new position. */
-    start_pos /= this->wave->nBlockAlign;
-    start_pos *= this->wave->nBlockAlign;
+    start_pos /= this->seek_align;
+    start_pos *= this->seek_align;
     start_pos += this->data_start;
 
     this->input->seek(this->input, start_pos, SEEK_SET);
@@ -336,7 +349,7 @@ static int demux_wav_seek (demux_plugin_t *this_gen,
 static void demux_wav_dispose (demux_plugin_t *this_gen) {
   demux_wav_t *this = (demux_wav_t *) this_gen;
 
-  free(this->wave);
+  _x_freep(&this->wave);
   free(this);
 }
 
@@ -354,11 +367,15 @@ static int demux_wav_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_wav_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_wav_get_optional_data(demux_plugin_t *this_gen,
 					void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -366,9 +383,22 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
                                     input_plugin_t *input) {
 
   demux_wav_t    *this;
-  uint32_t	align;
 
-  this         = calloc(1, sizeof(demux_wav_t));
+  switch (stream->content_detection_method) {
+    case METHOD_BY_MRL:
+    case METHOD_BY_CONTENT:
+    case METHOD_EXPLICIT:
+      if (!probe_wav_file(input))
+        return NULL;
+      break;
+    default:
+      return NULL;
+  }
+
+  this = calloc(1, sizeof(demux_wav_t));
+  if (!this)
+    return NULL;
+
   this->stream = stream;
   this->input  = input;
 
@@ -384,51 +414,31 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   this->status = DEMUX_FINISHED;
 
-  switch (stream->content_detection_method) {
-
-  case METHOD_BY_MRL:
-  case METHOD_BY_CONTENT:
-  case METHOD_EXPLICIT:
-
-    if (!open_wav_file(this)) {
-      free (this);
-      return NULL;
-    }
-
-  break;
-
-  default:
-    free (this);
+  if (!open_wav_file(this)) {
+    demux_wav_dispose(&this->demux_plugin);
     return NULL;
   }
 
-  /* special block alignment hack so that the demuxer doesn't send
-   * packets with individual PCM samples */
-  if ((this->wave->nAvgBytesPerSec / this->wave->nBlockAlign) ==
-      this->wave->nSamplesPerSec) {
-    align = PCM_BLOCK_ALIGN / this->wave->nBlockAlign;
-    align = align * this->wave->nBlockAlign;
-    this->wave->nBlockAlign = align;
-  }
-
   return &this->demux_plugin;
 }
 
-void *demux_wav_init_plugin (xine_t *xine, void *data) {
-  demux_wav_class_t     *this;
+void *demux_wav_init_plugin (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_wav_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("WAV file demux plugin");
-  this->demux_class.identifier      = "WAV";
-  this->demux_class.mimetypes       =
-    "audio/x-wav: wav: WAV audio;"
-    "audio/wav: wav: WAV audio;"
-    "audio/x-pn-wav: wav: WAV audio;"
-    "audio/x-pn-windows-acm: wav: WAV audio;";
-  this->demux_class.extensions      = "wav";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_wav_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("WAV file demux plugin"),
+    .identifier      = "WAV",
+    .mimetypes       =
+      "audio/x-wav: wav: WAV audio;"
+      "audio/wav: wav: WAV audio;"
+      "audio/x-pn-wav: wav: WAV audio;"
+      "audio/x-pn-windows-acm: wav: WAV audio;",
+    .extensions      = "wav",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_wav_class;
 }
