@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2008 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -25,7 +25,7 @@
 
 typedef struct rtsp_session_s rtsp_session_t;
 
-rtsp_session_t *rtsp_session_start(xine_stream_t *stream, char *mrl) XINE_MALLOC;
+rtsp_session_t *rtsp_session_start(xine_stream_t *stream, const char *mrl) XINE_MALLOC;
 
 void rtsp_session_set_start_time(rtsp_session_t *this, int start_time);
 
