@@ -48,18 +48,20 @@
 #define GREEDYTWOFRAMETHRESHOLD 4
 #define GREEDYTWOFRAMETHRESHOLD2 8
 
+#if defined(ARCH_X86)
 #define IS_MMXEXT 1
 #include "greedy2frame_template.c"
 #undef IS_MMXEXT
 
 #include "greedy2frame_template_sse2.c"
+#endif /* ARCH_X86 */
 
 static void DeinterlaceGreedy2Frame(uint8_t *output, int outstride,
                                     deinterlace_frame_data_t *data,
                                     int bottom_field, int second_field, int width, int height )
 
 {
-#if defined(ARCH_X86) || defined(ARCH_X86_64)
+#if defined(ARCH_X86)
 
     if (xine_mm_accel() & MM_ACCEL_X86_SSE2) {
         if (((uintptr_t)output & 15) || (outstride & 15) ||
@@ -88,7 +90,7 @@ static void DeinterlaceGreedy2Frame(uint8_t *output, int outstride,
 }
 
 
-static deinterlace_method_t greedy2framemethod =
+static const deinterlace_method_t greedy2framemethod =
 {
     "Greedy 2-frame (DScaler)",
     "Greedy2Frame",
@@ -103,7 +105,7 @@ static deinterlace_method_t greedy2framemethod =
     NULL
 };
 
-deinterlace_method_t *greedy2frame_get_method( void )
+const deinterlace_method_t *greedy2frame_get_method( void )
 {
     return &greedy2framemethod;
 }
