@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -42,19 +42,6 @@
 #include "deinterlace.h"
 #include "plugins/plugins.h"
 
-/* plugin class initialization function */
-static void *deinterlace_init_plugin(xine_t *xine, void *);
-
-
-/* plugin catalog information */
-static const post_info_t deinterlace_special_info = { XINE_POST_TYPE_VIDEO_FILTER };
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_POST | PLUGIN_MUST_PRELOAD, 10, "tvtime", XINE_VERSION_CODE, &deinterlace_special_info, &deinterlace_init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-
 
 typedef struct post_plugin_deinterlace_s post_plugin_deinterlace_t;
 
@@ -113,7 +100,6 @@ END_PARAM_DESCR( param_descr )
 
 typedef struct post_class_deinterlace_s {
   post_class_t class;
-  deinterlace_parameters_t init_param;
 
   deinterlace_methods_t    methods;
 } post_class_deinterlace_t;
@@ -121,7 +107,6 @@ typedef struct post_class_deinterlace_s {
 /* plugin structure */
 struct post_plugin_deinterlace_s {
   post_plugin_t      post;
-  xine_post_in_t     parameter_input;
 
   /* private data */
   int                cur_method;
@@ -161,9 +146,9 @@ static void _flush_frames(post_plugin_deinterlace_t *this)
   this->tvtime_changed++;
 }
 
-static int set_parameters (xine_post_t *this_gen, void *param_gen) {
+static int set_parameters (xine_post_t *this_gen, const void *param_gen) {
   post_plugin_deinterlace_t *this = (post_plugin_deinterlace_t *)this_gen;
-  deinterlace_parameters_t *param = (deinterlace_parameters_t *)param_gen;
+  const deinterlace_parameters_t *param = (const deinterlace_parameters_t *)param_gen;
 
   pthread_mutex_lock (&this->lock);
 
@@ -272,13 +257,6 @@ static char * get_help (void) {
   return (char *)help_string;
 }
 
-static xine_post_api_t post_api = {
-  set_parameters,
-  get_parameters,
-  get_param_descr,
-  get_help,
-};
-
 
 /* plugin class functions */
 static post_plugin_t *deinterlace_open_plugin(post_class_t *class_gen, int inputs,
@@ -303,7 +281,7 @@ static int            deinterlace_intercept_frame(post_video_port_t *port, vo_fr
 static int            deinterlace_draw(vo_frame_t *frame, xine_stream_t *stream);
 
 
-static void *deinterlace_init_plugin(xine_t *xine, void *data)
+static void *deinterlace_init_plugin(xine_t *xine, const void *data)
 {
   post_class_deinterlace_t *class = calloc(1, sizeof(post_class_deinterlace_t));
   uint32_t config_flags = xine_mm_accel();
@@ -312,6 +290,8 @@ static void *deinterlace_init_plugin(xine_t *xine, void *data)
   if (!class)
     return NULL;
 
+  (void)data;
+
   class->class.open_plugin     = deinterlace_open_plugin;
   class->class.identifier      = "tvtime";
   class->class.description     = N_("advanced deinterlacer plugin with pulldown detection");
@@ -361,18 +341,6 @@ static void *deinterlace_init_plugin(xine_t *xine, void *data)
   }
   enum_methods[i+1] = NULL;
 
-
-  /* Some default values */
-  class->init_param.method                     = 1; /* First (plugin) method available */
-  class->init_param.enabled                    = 1;
-  class->init_param.pulldown                   = 1; /* vektor */
-  class->init_param.pulldown_error_wait        = 60; /* about one second */
-  class->init_param.framerate_mode             = 0; /* full */
-  class->init_param.judder_correction          = 1;
-  class->init_param.use_progressive_frame_flag = 1;
-  class->init_param.chroma_filter              = 0;
-  class->init_param.cheap_mode                 = 0;
-
   return &class->class;
 }
 
@@ -383,26 +351,60 @@ static post_plugin_t *deinterlace_open_plugin(post_class_t *class_gen, int input
 {
   post_plugin_deinterlace_t *this = calloc(1, sizeof(post_plugin_deinterlace_t));
   post_in_t                 *input;
-  xine_post_in_t            *input_api;
   post_out_t                *output;
-  post_class_deinterlace_t  *class = (post_class_deinterlace_t *)class_gen;
   post_video_port_t *port;
+  tvtime_t *tvtime;
+
+  static const xine_post_api_t post_api = {
+    .set_parameters  = set_parameters,
+    .get_parameters  = get_parameters,
+    .get_param_descr = get_param_descr,
+    .get_help        = get_help,
+  };
+  static const xine_post_in_t params_input = {
+    .name = "parameters",
+    .type = XINE_POST_DATA_PARAMETERS,
+    .data = (void *)&post_api,
+  };
+
+  /* Some default values */
+  static const deinterlace_parameters_t init_param = {
+    .method                     = 1, /* First (plugin) method available */
+    .enabled                    = 1,
+    .pulldown                   = 1, /* vektor */
+    .pulldown_error_wait        = 60, /* about one second */
+    .framerate_mode             = 0, /* full */
+    .judder_correction          = 1,
+    .use_progressive_frame_flag = 1,
+    .chroma_filter              = 0,
+    .cheap_mode                 = 0,
+  };
 
   if (!this || !video_target || !video_target[0]) {
     free(this);
     return NULL;
   }
 
+  (void)class_gen;
+  (void)inputs;
+  (void)audio_target;
+
+  tvtime = tvtime_new_context();
+  if (!tvtime) {
+    free(this);
+    return NULL;
+  }
+
   _x_post_init(&this->post, 0, 1);
 
-  this->tvtime = tvtime_new_context();
+  this->tvtime = tvtime;
   this->tvtime_changed++;
   this->tvtime_last_filmmode = 0;
   this->class = (post_class_deinterlace_t *)class_gen;
 
   pthread_mutex_init (&this->lock, NULL);
 
-  set_parameters (&this->post.xine_post, &class->init_param);
+  set_parameters (&this->post.xine_post, &init_param);
 
   port = _x_post_intercept_video_port(&this->post, video_target[0], &input, &output);
   /* replace with our own get_frame function */
@@ -414,11 +416,7 @@ static post_plugin_t *deinterlace_open_plugin(post_class_t *class_gen, int input
   port->intercept_frame       = deinterlace_intercept_frame;
   port->new_frame->draw       = deinterlace_draw;
 
-  input_api       = &this->parameter_input;
-  input_api->name = "parameters";
-  input_api->type = XINE_POST_DATA_PARAMETERS;
-  input_api->data = &post_api;
-  xine_list_push_back(this->post.input, input_api);
+  xine_list_push_back(this->post.input, (void *)&params_input);
 
   input->xine_in.name     = "video";
   output->xine_out.name   = "deinterlaced video";
@@ -855,7 +853,7 @@ static int deinterlace_draw(vo_frame_t *frame, xine_stream_t *stream)
       if (this->recent_frame[0] && !this->recent_frame[0]->progressive_frame &&
           this->tvtime->curmethod->delaysfield)
       {
-	skip = deinterlace_build_output_field(
+        deinterlace_build_output_field(
           this, port, stream,
           frame, yuy2_frame,
           fields[0], 0,
@@ -937,3 +935,14 @@ static int deinterlace_draw(vo_frame_t *frame, xine_stream_t *stream)
 
   return skip;
 }
+
+/* plugin catalog information */
+static const post_info_t deinterlace_special_info = {
+  .type = XINE_POST_TYPE_VIDEO_FILTER,
+};
+
+const plugin_info_t xine_plugin_info[] EXPORTED = {
+  /* type, API, "name", version, special_info, init_function */
+  { PLUGIN_POST | PLUGIN_MUST_PRELOAD, 10, "tvtime", XINE_VERSION_CODE, &deinterlace_special_info, &deinterlace_init_plugin },
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
+};
