@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2018 the xine project
+ * Copyright (C) 2000-2021 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -40,6 +40,8 @@
 #include <xine/xineutils.h>
 #include <xine/osd.h>
 
+#include "xine-engine/bswap.h"
+
 #include "sputext_demuxer.h"
 
 #define SUB_MAX_TEXT  5      /* lines */
@@ -110,16 +112,14 @@ typedef enum {
   SUBTITLE_SIZE_NUM        /* number of values in enum */
 } subtitle_size;
 
-#define FONTNAME_SIZE 100
-
 typedef struct sputext_class_s {
   spu_decoder_class_t class;
 
   subtitle_size      subtitle_size;   /* size of subtitles */
   int                vertical_offset;
-  char               font[FONTNAME_SIZE]; /* subtitle font */
+  char              *font;            /* subtitle font */
 #ifdef HAVE_FT2
-  char               font_ft[FILENAME_MAX]; /* subtitle font */
+  char              *font_ft;         /* subtitle font */
   int                use_font_ft;     /* use Freetype */
 #endif
   const char        *src_encoding;    /* encoding of subtitle file */
@@ -127,6 +127,7 @@ typedef struct sputext_class_s {
 
   xine_t            *xine;
 
+  pthread_mutex_t    font_name_mutex;
 } sputext_class_t;
 
 
@@ -154,7 +155,7 @@ typedef struct sputext_decoder_s {
    */
   subtitle_size      subtitle_size;   /* size of subtitles */
   int                vertical_offset;
-  char               font[FILENAME_MAX]; /* subtitle font */
+  char              *font;            /* subtitle font */
   const char         *buf_encoding;    /* encoding of subtitle buffer */
 
   int                width;          /* frame width                */
@@ -179,15 +180,34 @@ typedef struct sputext_decoder_s {
   video2wnd_t        video2wnd;
 } sputext_decoder_t;
 
-static inline char *get_font (sputext_class_t *class)
+static inline int update_font (sputext_decoder_t *this)
 {
+  sputext_class_t *class = this->class;
+  char *font;
+  int result = 0;
+
+  pthread_mutex_lock(&class->font_name_mutex);
 #ifdef HAVE_FT2
-  return class->use_font_ft ? class->font_ft : class->font;
+  font = class->use_font_ft ? class->font_ft : class->font;
 #else
-  return class->font;
+  font = class->font;
 #endif
+  if (!this->font || strcmp(font, this->font)) {
+    free(this->font);
+    this->font = strdup(font);
+    result = 1;
+  }
+  pthread_mutex_unlock(&class->font_name_mutex);
+
+  return result;
 }
 
+static inline const char *get_font (sputext_decoder_t *this) {
+  update_font(this);
+  return this->font;
+}
+
+
 static void update_font_size (sputext_decoder_t *this, int force_update) {
   static const int sizes[SUBTITLE_SIZE_NUM] = { 16, 20, 24, 32, 48, 64 };
 
@@ -211,7 +231,7 @@ static void update_font_size (sputext_decoder_t *this, int force_update) {
                                             this->width,
                                             this->height);
 
-    this->renderer->set_font (this->osd, get_font (this->class), this->font_size);
+    this->renderer->set_font (this->osd, get_font (this), this->font_size);
 
     this->renderer->set_position (this->osd, 0, 0);
   }
@@ -229,10 +249,12 @@ static void update_output_size (sputext_decoder_t *this) {
 
   /* initialize decoder if needed */
   if( this->unscaled ) {
-    if( this->width != this->stream->video_out->get_property(this->stream->video_out,
-                                                             VO_PROP_WINDOW_WIDTH) ||
-        this->height != this->stream->video_out->get_property(this->stream->video_out,
-                                                             VO_PROP_WINDOW_HEIGHT) ||
+    int win_width = this->stream->video_out->get_property(this->stream->video_out,
+                                                          VO_PROP_WINDOW_WIDTH);
+    int win_height = this->stream->video_out->get_property(this->stream->video_out,
+                                                           VO_PROP_WINDOW_HEIGHT);
+
+    if( this->width != win_width || this->height != win_height ||
         !this->img_duration || !this->osd ) {
 
       int width = 0, height = 0;
@@ -241,10 +263,8 @@ static void update_output_size (sputext_decoder_t *this) {
                                       &width, &height, &this->img_duration );
       if( width && height ) {
 
-        this->width = this->stream->video_out->get_property(this->stream->video_out,
-                                                             VO_PROP_WINDOW_WIDTH);
-        this->height = this->stream->video_out->get_property(this->stream->video_out,
-                                                             VO_PROP_WINDOW_HEIGHT);
+        this->width = win_width;
+        this->height = win_height;
 
         if(!this->osd || (this->width && this->height)) {
 
@@ -410,7 +430,7 @@ static int ogm_render_line_internal(sputext_decoder_t *this, int x, int y, const
     }
 
     char letter[5];
-    const char *const encoding = this->buf_encoding ? : this->class->src_encoding;
+    const char *const encoding = this->buf_encoding ? this->buf_encoding : this->class->src_encoding;
     const int isutf8 = !strcmp(encoding, "utf-8");
     const size_t shift = isutf8 ? parse_utf8_size (&text[i]) : 1;
     memcpy(letter,&text[i],shift);
@@ -566,20 +586,24 @@ static void draw_subtitle(sputext_decoder_t *this, int64_t sub_start, int64_t su
   if ( ! this->renderer )
     return;
 
+  if( this->last_subtitle_end && sub_start < this->last_subtitle_end ) {
+    sub_start = this->last_subtitle_end;
+    if (sub_start >= sub_end)
+	return;
+  }
+  this->last_subtitle_end = sub_end;
+
   read_ssa_tag(this, this->text[0], &alignment, &sub_x, &sub_y, &max_width);
 
   update_font_size(this, 0);
 
-  const char *const font = get_font (this->class);
-  if( strcmp(this->font, font) ) {
-    strncpy(this->font, font, FILENAME_MAX);
-    this->font[FILENAME_MAX - 1] = '\0';
-    this->renderer->set_font (this->osd, font, this->font_size);
+  if( update_font(this)) {
+    this->renderer->set_font (this->osd, this->font, this->font_size);
   }
 
   int font_size = this->font_size;
 
-  const char *const encoding = this->buf_encoding ? : this->class->src_encoding;
+  const char *const encoding = this->buf_encoding ? this->buf_encoding : this->class->src_encoding;
   this->renderer->set_encoding(this->osd, encoding);
 
   int rebuild_all = 0;
@@ -649,7 +673,7 @@ static void draw_subtitle(sputext_decoder_t *this, int64_t sub_start, int64_t su
       if (len)
         buf[len] = ' ';
 
-      strncat(buf, this->text[line], SUB_BUFSIZE-len-1);
+      strlcat(buf, this->text[line], sizeof(this->text[0]));
     }
 
     char *stream = buf;
@@ -687,8 +711,7 @@ static void draw_subtitle(sputext_decoder_t *this, int64_t sub_start, int64_t su
 
       /* copy current line */
       if (best_cut != NULL) *best_cut = 0;
-      strncpy(this->text[this->lines], stream, SUB_BUFSIZE);
-      this->text[this->lines][SUB_BUFSIZE - 1] = 0;
+      strlcpy(this->text[this->lines], stream, sizeof(this->text[0]));
       this->lines = this->lines + 1;
 
       stream = best_cut + 1;
@@ -756,7 +779,7 @@ static void draw_subtitle(sputext_decoder_t *this, int64_t sub_start, int64_t su
 
       if( w > max_width && font_size > 16 ) {
         font_size -= 4;
-        this->renderer->set_font (this->osd, get_font (this->class), font_size);
+        this->renderer->set_font (this->osd, get_font (this), font_size);
       } else {
         break;
       }
@@ -771,12 +794,7 @@ static void draw_subtitle(sputext_decoder_t *this, int64_t sub_start, int64_t su
   }
 
   if( font_size != this->font_size )
-    this->renderer->set_font (this->osd, get_font (this->class), this->font_size);
-
-  if( this->last_subtitle_end && sub_start < this->last_subtitle_end ) {
-    sub_start = this->last_subtitle_end;
-  }
-  this->last_subtitle_end = sub_end;
+    this->renderer->set_font (this->osd, get_font (this), this->font_size);
 
   this->renderer->set_text_palette (this->osd, -1, OSD_TEXT1);
   this->renderer->get_palette(this->osd, this->spu_palette, this->spu_trans);
@@ -806,7 +824,6 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
   int64_t start_vpts, end_vpts;
   int64_t spu_offset;
   int i;
-  uint32_t *val;
   char *str;
   extra_info_t extra_info;
   int master_status, slave_status;
@@ -819,9 +836,6 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
   if (buf->decoder_flags & BUF_FLAG_PREVIEW)
     return;
 
-  if ((this->stream->spu_channel & 0x1f) != (buf->type & 0x1f))
-    return;
-
   if ( (buf->decoder_flags & BUF_FLAG_SPECIAL) &&
        (buf->decoder_info[1] == BUF_SPECIAL_CHARSET_ENCODING) )
     this->buf_encoding = buf->decoder_info_ptr[2];
@@ -834,10 +848,11 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
 
     this->ogm = 1;
     uses_time = 1;
-    val = (uint32_t * )buf->content;
-    start = *val++;
-    end = *val++;
-    str = (char *)val;
+
+    start = _X_ME_32(buf->content);
+    end   = _X_ME_32(buf->content + 4);
+
+    str = buf->content + 8;
 
     if (!*str) return;
     /* Empty ogm packets (as created by ogmmux) clears out old messages. We already respect the end time. */
@@ -870,16 +885,16 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
   } else {
 
     this->ogm = 0;
-    val = (uint32_t * )buf->content;
 
-    this->lines = *val++;
-    uses_time = *val++;
-    start = *val++;
-    end = *val++;
-    str = (char *)val;
+    this->lines = _X_ME_32(buf->content);
+    uses_time   = _X_ME_32(buf->content + 4);
+    start       = _X_ME_32(buf->content + 8);
+    end         = _X_ME_32(buf->content + 12);
+
+    str = buf->content + 16;
+
     for (i = 0; i < this->lines; i++, str += strlen(str) + 1) {
-      strncpy( this->text[i], str, SUB_BUFSIZE - 1);
-      this->text[i][SUB_BUFSIZE - 1] = '\0';
+      strlcpy( this->text[i], str, sizeof(this->text[i]));
     }
 
   }
@@ -987,8 +1002,12 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
         }
 
         _x_spu_decoder_sleep(this->stream, start_vpts);
-        update_output_size( this );
-        draw_subtitle(this, start_vpts, end_vpts);
+
+	if (this->stream->spu_channel >= 0 && (this->stream->spu_channel & 0x1f) == (buf->type & 0x1f))
+	{
+	    update_output_size( this );
+	    draw_subtitle(this, start_vpts, end_vpts);
+	}
 
         return;
       }
@@ -1023,6 +1042,7 @@ static void spudec_dispose (spu_decoder_t *this_gen) {
     this->renderer->free_object (this->osd);
     this->osd = NULL;
   }
+  _x_freep(&this->font);
   free(this);
 }
 
@@ -1037,8 +1057,10 @@ static void update_osd_font(void *class_gen, xine_cfg_entry_t *entry)
 {
   sputext_class_t *class = (sputext_class_t *)class_gen;
 
-  strncpy(class->font, entry->str_value, FONTNAME_SIZE);
-  class->font[FONTNAME_SIZE - 1] = '\0';
+  pthread_mutex_lock(&class->font_name_mutex);
+  free(class->font);
+  class->font = strdup(entry->str_value);
+  pthread_mutex_unlock(&class->font_name_mutex);
 
   xprintf(class->xine, XINE_VERBOSITY_DEBUG, "libsputext: spu_font = %s\n", class->font );
 }
@@ -1048,8 +1070,10 @@ static void update_osd_font_ft(void *class_gen, xine_cfg_entry_t *entry)
 {
   sputext_class_t *class = (sputext_class_t *)class_gen;
 
-  strncpy(class->font_ft, entry->str_value, FILENAME_MAX);
-  class->font_ft[FILENAME_MAX - 1] = '\0';
+  pthread_mutex_lock(&class->font_name_mutex);
+  free(class->font_ft);
+  class->font_ft = strdup(entry->str_value);
+  pthread_mutex_unlock(&class->font_name_mutex);
 
   xprintf(class->xine, XINE_VERBOSITY_DEBUG, "libsputext: spu_font_ft = %s\n", class->font_ft);
 }
@@ -1097,20 +1121,19 @@ static spu_decoder_t *sputext_class_open_plugin (spu_decoder_class_t *class_gen,
   this->class  = class;
   this->stream = stream;
 
-  return (spu_decoder_t *) this;
+  return &this->spu_decoder;
 }
 
 static void sputext_class_dispose (spu_decoder_class_t *class_gen) {
   sputext_class_t *this = (sputext_class_t *)class_gen;
 
-  this->xine->config->unregister_callback(this->xine->config,
-					  "subtitles.separate.src_encoding");
-  this->xine->config->unregister_callback(this->xine->config,
-					  "subtitles.separate.subtitle_size");
-  this->xine->config->unregister_callback(this->xine->config,
-					  "subtitles.separate.vertical_offset");
-  this->xine->config->unregister_callback(this->xine->config,
-					  "subtitles.separate.use_unscaled_osd");
+  this->xine->config->unregister_callbacks (this->xine->config, NULL, NULL, this, sizeof (*this));
+
+  pthread_mutex_destroy(&this->font_name_mutex);
+  _x_freep(&this->font);
+#ifdef HAVE_FT2
+  _x_freep(&this->font_ft);
+#endif
   free (this);
 }
 
@@ -1144,6 +1167,8 @@ static void *init_spu_decoder_plugin (xine_t *xine, const void *data) {
 
   this->xine                   = xine;
 
+  pthread_mutex_init(&this->font_name_mutex, NULL);
+
   this->subtitle_size  = xine->config->register_enum(xine->config,
 			      "subtitles.separate.subtitle_size",
 			       1,
@@ -1159,22 +1184,20 @@ static void *init_spu_decoder_plugin (xine_t *xine, const void *data) {
 			      _("You can adjust the vertical position of the subtitle. "
 			        "The setting will be evaluated relative to the window size."),
 			      0, update_vertical_offset, this);
-  strncpy(this->font, xine->config->register_string(xine->config,
+  this->font = strdup(xine->config->register_string(xine->config,
 				"subtitles.separate.font",
 				"sans",
 				_("font for subtitles"),
 				_("A font from the xine font directory to be used for the "
 				  "subtitle text."),
-				10, update_osd_font, this), FONTNAME_SIZE);
-  this->font[FONTNAME_SIZE - 1] = '\0';
+				10, update_osd_font, this));
 #ifdef HAVE_FT2
-  strncpy(this->font_ft, xine->config->register_filename(xine->config,
+  this->font_ft = strdup(xine->config->register_filename(xine->config,
 				"subtitles.separate.font_freetype",
 				"", XINE_CONFIG_STRING_IS_FILENAME,
 				_("font for subtitles"),
 				_("An outline font file (e.g. a .ttf) to be used for the subtitle text."),
-				10, update_osd_font_ft, this), FILENAME_MAX);
-  this->font_ft[FILENAME_MAX - 1] = '\0';
+				10, update_osd_font_ft, this));
   this->use_font_ft = xine->config->register_bool(xine->config,
 				"subtitles.separate.font_use_freetype",
 				0,
