@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2017 the xine project
+ * Copyright (C) 2008-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -21,6 +21,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "group_vdpau.h"
 
 /*
@@ -29,7 +33,6 @@
  * list of valid buffer types (and add a new one if the one you need does
  * not exist). Terminate the list with a 0.
  */
-static const uint32_t video_types_alterh264[] = { BUF_VIDEO_H264, 0 };
 static const uint32_t video_types_h264[]      = { BUF_VIDEO_H264, 0 };
 static const uint32_t video_types_vc1[]       = { BUF_VIDEO_VC1, BUF_VIDEO_WMV9, 0 };
 static const uint32_t video_types_mpeg12[]    = { BUF_VIDEO_MPEG, 0 };
@@ -41,11 +44,11 @@ static const uint32_t video_types_mpeg4[]     = { BUF_VIDEO_MPEG4, BUF_VIDEO_XVI
  * plugins that handle the same buffer type. A plugin with priority (n+1)
  * will be used instead of a plugin with priority (n).
  */
-static const decoder_info_t dec_info_video_alterh264 = { video_types_alterh264, 9 };
-static const decoder_info_t dec_info_video_h264      = { video_types_h264,      7 };
-static const decoder_info_t dec_info_video_vc1       = { video_types_vc1,       8 };
-static const decoder_info_t dec_info_video_mpeg12    = { video_types_mpeg12,    8 };
-static const decoder_info_t dec_info_video_mpeg4     = { video_types_mpeg4,     8 };
+static const decoder_info_t dec_info_video_alterh264 = { .supported_types = video_types_h264,   .priority = 9 };
+static const decoder_info_t dec_info_video_h264      = { .supported_types = video_types_h264,   .priority = 7 };
+static const decoder_info_t dec_info_video_vc1       = { .supported_types = video_types_vc1,    .priority = 8 };
+static const decoder_info_t dec_info_video_mpeg12    = { .supported_types = video_types_mpeg12, .priority = 8 };
+static const decoder_info_t dec_info_video_mpeg4     = { .supported_types = video_types_mpeg4,  .priority = 8 };
 
 /*
  * The plugin catalog entry. This is the only information that a plugin
@@ -63,6 +66,6 @@ const plugin_info_t xine_plugin_info[] EXPORTED = {
     19, "vdpau_mpeg12",     XINE_VERSION_CODE, &dec_info_video_mpeg12,    mpeg12_init_plugin },
   { PLUGIN_VIDEO_DECODER,
     19, "vdpau_mpeg4",      XINE_VERSION_CODE, &dec_info_video_mpeg4,     mpeg4_init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 
