@@ -1,6 +1,6 @@
 /*
  * kate: space-indent on; indent-width 2; mixedindent off; indent-mode cstyle; remove-trailing-space on;
- * Copyright (C) 2008-2017 the xine project
+ * Copyright (C) 2008-2020 the xine project
  * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
  *
  * This file is part of xine, a free video player.
@@ -172,17 +172,9 @@ typedef struct {
 } sequence_t;
 
 
-
-typedef struct {
-  video_decoder_class_t   decoder_class;
-} vdpau_mpeg12_class_t;
-
-
-
 typedef struct vdpau_mpeg12_decoder_s {
   video_decoder_t         video_decoder;  /* parent video decoder structure */
 
-  vdpau_mpeg12_class_t    *class;
   xine_stream_t           *stream;
 
   sequence_t              sequence;
@@ -693,11 +685,15 @@ static void decode_render( vdpau_mpeg12_decoder_t *vd, vdpau_accel_t *accel )
 
   VdpStatus st;
   if ( vd->decoder==VDP_INVALID_HANDLE || vd->decoder_profile!=seq->profile || vd->decoder_width!=seq->coded_width || vd->decoder_height!=seq->coded_height ) {
+    if (accel->lock)
+      accel->lock (accel->vo_frame);
     if ( vd->decoder!=VDP_INVALID_HANDLE ) {
       accel->vdp_decoder_destroy( vd->decoder );
       vd->decoder = VDP_INVALID_HANDLE;
     }
     st = accel->vdp_decoder_create( accel->vdp_device, seq->profile, seq->coded_width, seq->coded_height, 2, &vd->decoder);
+    if (accel->unlock)
+      accel->unlock (accel->vo_frame);
     if ( st!=VDP_STATUS_OK )
       lprintf( "failed to create decoder !! %s\n", accel->vdp_get_error_string( st ) );
     else {
@@ -712,12 +708,16 @@ static void decode_render( vdpau_mpeg12_decoder_t *vd, vdpau_accel_t *accel )
   vbit.struct_version = VDP_BITSTREAM_BUFFER_VERSION;
   vbit.bitstream = pic->slices;
   vbit.bitstream_bytes = (pic->vdp_infos.picture_structure==PICTURE_FRAME)? pic->slices_pos : pic->slices_pos_top;
+  if (accel->lock)
+    accel->lock (accel->vo_frame);
   st = accel->vdp_decoder_render( vd->decoder, accel->surface, CAST_VdpPictureInfo_PTR &pic->vdp_infos, 1, &vbit );
+  if (accel->unlock)
+    accel->unlock (accel->vo_frame);
 #ifdef LOG
   if ( st!=VDP_STATUS_OK )
     lprintf( "decoder failed : %d!! %s\n", st, accel->vdp_get_error_string( st ) );
   else {
-    lprintf( "DECODER SUCCESS : frame_type:%d, slices=%d, slices_bytes=%d, current=%d, forwref:%d, backref:%d, pts:%lld\n",
+    lprintf( "DECODER SUCCESS : frame_type:%d, slices=%d, slices_bytes=%d, current=%d, forwref:%d, backref:%d, pts:%"PRId64"\n",
       pic->vdp_infos.picture_coding_type, pic->vdp_infos.slice_count, vbit.bitstream_bytes, accel->surface, pic->vdp_infos.forward_reference, pic->vdp_infos.backward_reference, seq->cur_pts );
     VdpPictureInfoMPEG1Or2 *info = &pic->vdp_infos;
     lprintf("%d %d %d %d %d %d %d %d %d %d %d %d %d\n", info->intra_dc_precision, info->frame_pred_frame_dct, info->concealment_motion_vectors,
@@ -742,7 +742,11 @@ static void decode_render( vdpau_mpeg12_decoder_t *vd, vdpau_accel_t *accel )
     vbit.struct_version = VDP_BITSTREAM_BUFFER_VERSION;
     vbit.bitstream = pic->slices+pic->slices_pos_top;
     vbit.bitstream_bytes = pic->slices_pos-pic->slices_pos_top;
+    if (accel->lock)
+      accel->lock (accel->vo_frame);
     st = accel->vdp_decoder_render( vd->decoder, accel->surface, CAST_VdpPictureInfo_PTR &pic->vdp_infos2, 1, &vbit );
+    if (accel->unlock)
+      accel->unlock (accel->vo_frame);
     if ( st!=VDP_STATUS_OK )
       lprintf( "decoder failed : %d!! %s\n", st, accel->vdp_get_error_string( st ) );
     else
@@ -928,14 +932,14 @@ static void vdpau_mpeg12_decode_data (video_decoder_t *this_gen, buf_element_t *
     seq->cur_pts = buf->pts;
 
   int size = seq->bufpos+buf->size;
-  if ( seq->bufsize < size ) {
+  if ( seq->bufsize < (unsigned int)size ) {
     seq->bufsize = size+1024;
     seq->buf = realloc( seq->buf, seq->bufsize );
   }
   xine_fast_memcpy( seq->buf+seq->bufpos, buf->content, buf->size );
   seq->bufpos += buf->size;
 
-  while ( seq->bufseek <= seq->bufpos-4 ) {
+  while ( seq->bufseek <= (int)(seq->bufpos)-4 ) {
     uint8_t *buffer = seq->buf+seq->bufseek;
     if ( buffer[0]==0 && buffer[1]==0 && buffer[2]==1 ) {
       if ( seq->start<0 ) {
@@ -969,6 +973,7 @@ static void vdpau_mpeg12_decode_data (video_decoder_t *this_gen, buf_element_t *
 static void vdpau_mpeg12_flush (video_decoder_t *this_gen) {
 
   lprintf( "vdpau_mpeg12_flush\n" );
+  (void)this_gen;
 }
 
 /*
@@ -1001,8 +1006,12 @@ static void vdpau_mpeg12_dispose (video_decoder_t *this_gen) {
   lprintf( "vdpau_mpeg12_dispose\n" );
 
   if ( this->decoder!=VDP_INVALID_HANDLE && this->sequence.accel_vdpau ) {
+      if (this->sequence.accel_vdpau->lock)
+        this->sequence.accel_vdpau->lock (this->sequence.accel_vdpau->vo_frame);
       this->sequence.accel_vdpau->vdp_decoder_destroy( this->decoder );
       this->decoder = VDP_INVALID_HANDLE;
+      if (this->sequence.accel_vdpau->unlock)
+        this->sequence.accel_vdpau->unlock (this->sequence.accel_vdpau->vo_frame);
     }
 
   free_sequence( &this->sequence );
@@ -1024,25 +1033,41 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
 
   lprintf( "open_plugin\n" );
 
+  (void)class_gen;
+
   /* the videoout must be vdpau-capable to support this decoder */
-  if ( !(stream->video_driver->get_capabilities(stream->video_driver) & VO_CAP_VDPAU_MPEG12) )
+  if ( !(stream->video_out->get_capabilities(stream->video_out) & VO_CAP_VDPAU_MPEG12) )
     return NULL;
 
   /* now check if vdpau has free decoder resource */
-  vo_frame_t *img = stream->video_out->get_frame( stream->video_out, 1920, 1080, 1, XINE_IMGFMT_VDPAU, VO_BOTH_FIELDS );
+  vo_frame_t *img = stream->video_out->get_frame( stream->video_out, 1920, 1080, 1, XINE_IMGFMT_VDPAU,
+                                                  VO_BOTH_FIELDS | VO_GET_FRAME_MAY_FAIL );
+  if (!img) {
+    return NULL;
+  }
   vdpau_accel_t *accel = (vdpau_accel_t*)img->accel_data;
   int runtime_nr = accel->vdp_runtime_nr;
   img->free(img);
   VdpDecoder decoder;
+  if (accel->lock)
+    accel->lock (accel->vo_frame);
   VdpStatus st = accel->vdp_decoder_create( accel->vdp_device, VDP_DECODER_PROFILE_MPEG2_MAIN, 1920, 1080, 2, &decoder );
+  if (accel->unlock)
+    accel->unlock (accel->vo_frame);
   if ( st!=VDP_STATUS_OK ) {
     lprintf( "can't create vdpau decoder.\n" );
     return NULL;
   }
 
+  if (accel->lock)
+    accel->lock (accel->vo_frame);
   accel->vdp_decoder_destroy( decoder );
+  if (accel->unlock)
+    accel->unlock (accel->vo_frame);
 
   this = (vdpau_mpeg12_decoder_t *) calloc(1, sizeof(vdpau_mpeg12_decoder_t));
+  if (!this)
+    return NULL;
 
   this->video_decoder.decode_data         = vdpau_mpeg12_decode_data;
   this->video_decoder.flush               = vdpau_mpeg12_flush;
@@ -1051,7 +1076,6 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
   this->video_decoder.dispose             = vdpau_mpeg12_dispose;
 
   this->stream                            = stream;
-  this->class                             = (vdpau_mpeg12_class_t *) class_gen;
 
   this->sequence.bufsize = 1024;
   this->sequence.buf = (uint8_t*)malloc(this->sequence.bufsize);
@@ -1081,19 +1105,20 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
  * This function allocates a private video decoder class and initializes
  * the class's member functions.
  */
-void *mpeg12_init_plugin (xine_t *xine, void *data) {
+void *mpeg12_init_plugin (xine_t *xine, const void *data) {
 
-  vdpau_mpeg12_class_t *this;
+  (void)xine;
+  (void)data;
 
-  this = (vdpau_mpeg12_class_t *) calloc(1, sizeof(vdpau_mpeg12_class_t));
+  static const video_decoder_class_t decode_video_vdpau_mpeg12_class = {
+    .open_plugin     = open_plugin,
+    .identifier      = "vdpau_mpeg12",
+    .description     =
+        N_("vdpau_mpeg12: mpeg1/2 decoder plugin using VDPAU hardware decoding.\n"
+           "Must be used along with video_out_vdpau."),
+    .dispose         = NULL,
+  };
 
-  this->decoder_class.open_plugin     = open_plugin;
-  this->decoder_class.identifier      = "vdpau_mpeg12";
-  this->decoder_class.description     =
-	N_("vdpau_mpeg12: mpeg1/2 decoder plugin using VDPAU hardware decoding.\n"
-	   "Must be used along with video_out_vdpau.");
-  this->decoder_class.dispose         = default_video_decoder_class_dispose;
-
-  return this;
+  return (void *)&decode_video_vdpau_mpeg12_class;
 }
 
