@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2020 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -69,13 +69,15 @@ static void update_spu_decoder (xine_stream_t *stream, int type) {
 
 int _x_spu_decoder_sleep (xine_stream_t *s, int64_t next_spu_vpts) {
   xine_stream_private_t *stream = (xine_stream_private_t *)s;
-  xine_private_t *xine = (xine_private_t *)stream->s.xine;
+  xine_private_t *xine;
   int64_t time, wait;
   int thread_vacant = 1;
 
   if (!stream)
     return 0;
 
+  xine = (xine_private_t *)stream->s.xine;
+
   /* we wait until one second before the next SPU is due */
   next_spu_vpts -= 90000;
 
@@ -112,7 +114,7 @@ int _x_spu_decoder_sleep (xine_stream_t *s, int64_t next_spu_vpts) {
 
 static void *video_decoder_loop (void *stream_gen) {
 
-  xine_stream_private_t *stream = (xine_stream_private_t *)stream_gen;
+  xine_stream_private_t *stream = (xine_stream_private_t *)stream_gen, *m = stream->side_streams[0];
   xine_private_t *xine = (xine_private_t *)stream->s.xine;
   xine_ticket_t   *running_ticket = xine->port_ticket;
   int              running = 1;
@@ -121,6 +123,7 @@ static void *video_decoder_loop (void *stream_gen) {
   int              prof_video_decode = -1;
   int              prof_spu_decode = -1;
   uint32_t         buftype_unknown = 0;
+  int64_t          last_pts = 0;
   /* generic bitrate estimation. */
   int64_t          video_br_lasttime = 0;
   uint32_t         video_br_lastsize = 0;
@@ -161,7 +164,7 @@ static void *video_decoder_loop (void *stream_gen) {
 
     buf = stream->s.video_fifo->tget (stream->s.video_fifo, running_ticket);
 
-    _x_extra_info_merge( stream->video_decoder_extra_info, buf->extra_info );
+    _x_extra_info_merge (stream->video_decoder_extra_info, buf->extra_info);
     stream->video_decoder_extra_info->seek_count = stream->video_seek_count;
 
     lprintf ("got buffer 0x%08x\n", buf->type);
@@ -227,7 +230,18 @@ static void *video_decoder_loop (void *stream_gen) {
           stream->stream_info[XINE_STREAM_INFO_VIDEO_HANDLED] = handled;
           xine_rwlock_unlock (&stream->info_lock);
         }
-
+        /* legacy: decoders need not pass extra info themselves.
+         * pass them around here, and allow video_out.vo_set_img_ei () to fix reordering.
+         * both will be called from this thread (directly below and inside video_port.draw ()). */
+        if (buf->pts && (buf->pts != last_pts)) {
+          last_pts = buf->pts;
+          m->video_decoder_ei_index = (m->video_decoder_ei_index + 1) & (_XINE_EI_RING_SIZE - 1);
+          /* this covers roughly 1.4 seconds at 5.7 ms resolution, which alone should be
+           * enough to identify the ring index used. */
+          m->video_decoder_ei_fast[((uint32_t)last_pts >> 9) & 255] = m->video_decoder_ei_index;
+          m->ei[2 + m->video_decoder_ei_index].pts = last_pts;
+          m->ei[2 + m->video_decoder_ei_index].ei  = *stream->video_decoder_extra_info;
+        }
         /* video_br_add. some decoders reset buf->pts, do this first. */
         if (buf->pts) {
           int64_t d = buf->pts - video_br_lasttime;
@@ -692,3 +706,4 @@ void _x_video_decoder_shutdown (xine_stream_t *s) {
     stream->s.video_fifo = NULL;
   }
 }
+
